;-------------------------------------------------------------    
;+    
; NAME:    
;       FPRAEFIX    
; PURPOSE:    
;       Returnes string containing name without the extension of input filename-string    
; CATEGORY:    
;       String-processing    
; CALLING SEQUENCE:    
;       result = FPRAEFIX(filename[,NO_PATH])    
; INPUTS:    
;       filename = String containing the full filename    
; KEYWORD PARAMETERS:    
;       NO_PATH = Return String without path    
; OUTPUTS:    
;       result = String containing the praefix as defined by the first substrings of 'filename' divided by '.' without the last)    
; COMMON BLOCKS:    
; NOTES:    
;       if there is no extension, the input filename is returned    
; MODIFICATION HISTORY:    
;       Matthias Liebetanz, TU-Darmstadt, Germany, 12 April, 1997.    
;    
;       No warranty that this program is free of error
;-    
;-------------------------------------------------------------    
FUNCTION FPRAEFIX,file,NO_PATH=NO_PATH    
    
IF STRPOS(file,'.') EQ -1 THEN RETURN,file $             ; Point was not found -> no extension    
ELSE BEGIN    
seperated=STR_SEP(file,'.')    
nstrings=N_ELEMENTS(seperated)    
praefix=seperated(0)    
FOR i=1,nstrings-2 DO praefix=praefix+'.'+seperated(i)    
IF KEYWORD_SET(NO_PATH) THEN BEGIN    
   CASE !VERSION.OS OF    
        'windows': BEGIN                                 ;IDL below version 4.0  
                 endpath=RSTRPOS(praefix,'\')    
                 IF endpath NE -1 THEN praefix=STRMID(praefix,endpath+1,255)    
                 END    
        'Win32': BEGIN                                   ;IDL since version 4.0  
                 endpath=RSTRPOS(praefix,'\')    
                 IF endpath NE -1 THEN praefix=STRMID(praefix,endpath+1,255)    
                 END    
        'MacOS': BEGIN    
                 endpath=RSTRPOS(praefix,'\')    
                 IF endpath NE -1 THEN praefix=STRMID(praefix,endpath+1,255)    
                 END    
          'vms': mssg=WIDGET_MESSAGE('Sorry, vms is unsopported for FPRAEFIX.')    
         ELSE: BEGIN    
                 endpath=RSTRPOS(praefix,'/')    
                 IF endpath NE -1 THEN praefix=STRMID(praefix,endpath+1,255)    
                 END    
   ENDCASE    
ENDIF    
RETURN, praefix    
ENDELSE    
END    
