;-------------------------------------------------------------
;+
; NAME:
;       GRIDAREA
; PURPOSE:
;       This routine visualizes earthquakes and more
; MAIN PROGRAM:
;       SLAP - program for testing SEISMOLAP 
;       SEISMOLAP was developed by Jochen Zschau,
;       Geoforschungszentrum Potsdam, Germany
; CALLING SEQUENCE:
;       result=GRIDAREA(GROUP=Group,lat,lon,LIMITS=limits,PLOTPOS=plotpos,TITLE=title,XYSIZE=xysize,GRID=grid)
; INPUTS:
; KEYWORD PARAMETERS:
;       LIMITS = Vewktor of user-defined limits=[xmin,xmax,ymin,ymax]
;                If LIMITS=limits is set, but limits is undefined, the borders of
;                the earthquakes are returned
; MODIFICATION HISTORY:
;       Matthias Liebetanz, TH-Darmstadt, Germany, 17. Apr 1998 .
;
;       No warranty that this program is free of error
;-
;-------------------------------------------------------------
FUNCTION GRIDAREA,GROUP=Group,$
               quake_x,$
               quake_y,$
               QUAKE_Z=quake_z,$
               QUAKE_STRENGTH=quake_strength,$
               QUAKE_JS=quake_js,$
               INFO_SHOCK=info_shock,$
               SELECTED_SCHOCK=selected_shock,$
               FOCUS=focus,$
               LIMITS=limits,$
               PLOTPOS=plotpos,$
               TITLE=title,$
               XYSIZE=xysize,$
               AREA=area,$
               GRID_X=grid_x,$
               GRID_Y=grid_y,$
               POINT_MODE=POINT_MODE, $
               SHOCKINFOTEXT=shockinfotext,$
               SUPRESS_SWITCH=SUPRESS_SWITCH,$
               SUPRESS_SELECT=SUPRESS_SELECT,$
               SUPRESS_GRID=SUPRESS_GRID,$
               MAINSHOCKS=mainshocks

  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

  junk   = { CW_PDMENU_S, flags:0, name:'' }

  IF NOT KEYWORD_SET(TITLE) THEN title='Grid-Area'
  IF NOT KEYWORD_SET(XYSIZE) THEN xysize=[750,535]
  IF NOT KEYWORD_SET(PLOTPOS) THEN plotpos=[100, 100, xysize(0)-50,xysize(1)-50]

  IF KEYWORD_SET(LIMITS) THEN BEGIN
     IF N_ELEMENTS(limits) NE 4 THEN BEGIN
        PRINT,'LIMITS must have 4 elements'
        RETURN,-1
     ENDIF
  ENDIF ELSE limits=[MIN(quake_x),MAX(quake_x),MIN(quake_y),MAX(quake_y)]

  IF KEYWORD_SET(AREA) THEN BEGIN
     IF N_ELEMENTS(area) NE 4 THEN BEGIN
        PRINT,'AREA must have 4 elements'
        RETURN,-1
     ENDIF
  ENDIF

  GRIDAREA = WIDGET_BASE(GROUP_LEADER=Group, $
      COLUMN=1, $
      MAP=1, $
      UVALUE='GRIDAREA',$
      TITLE=title)

  DRAW = WIDGET_DRAW( GRIDAREA, $
      BUTTON_EVENTS=0, $
      MOTION_EVENTS=0, $
      RETAIN=2, $
      UVALUE='DRAWGRIDAREA', $
      XSIZE=xysize(0), $
      YSIZE=xysize(1))

  BUTTONBASE= WIDGET_BASE(GRIDAREA,$
      /ROW, $
      MAP=1, $
      UVALUE='BUTTONBASE')

  IF NOT KEYWORD_SET(SUPRESS_SELECT) THEN BEGIN
     BASE1 = WIDGET_BASE(BUTTONBASE, $
         ROW=1, $
         MAP=1, $
         UVALUE='BASE')


     IF KEYWORD_SET(POINT_MODE) THEN value='Get Point' ELSE value='Select Area'
     SELECT_AREA = WIDGET_BUTTON( BASE1, $
         UVALUE='Select Area', $
         VALUE=value)

     CANCEL = WIDGET_BUTTON( BASE1, $
         UVALUE='Cancel', $
         VALUE='Cancel')
  ENDIF

  IF KEYWORD_SET(MAINSHOCKS) THEN BEGIN
     PICK_MAINSHOCK = WIDGET_BUTTON( BUTTONBASE, $
         UVALUE='PICK_MAINSHOCK', $
         VALUE='Get main event information')

    xmain=DBLARR(mainshocks)
    ymain=xmain

    strength=quake_strength
    info_shock=CREATE_STRUCT('mag',FLTARR(mainshocks), $
                             'lon',DBLARR(mainshocks), $
                             'lat',DBLARR(mainshocks), $
                             'z',DBLARR(mainshocks), $
                             'js',DBLARR(mainshocks) )

    FOR i=0,mainshocks-1 DO BEGIN
        mainshock=MAX(strength)
        imainshock=WHERE(strength EQ mainshock)
        xmain(i)=quake_x(imainshock(0))
        ymain(i)=quake_y(imainshock(0))
        info_shock.mag(i)=strength(imainshock(0))
        info_shock.lon(i)=quake_x(imainshock(0))
        info_shock.lat(i)=quake_y(imainshock(0))
        info_shock.z(i)=quake_z(imainshock(0))
        info_shock.js(i)=quake_js(imainshock(0))
        strength(imainshock(0))=0.
    ENDFOR    ;Search next event
  ENDIF



  BASE2 = WIDGET_BASE(BUTTONBASE, $
      ROW=1, $
      MAP=1, $
      UVALUE='BASE')

  OK = WIDGET_BUTTON( BASE2, $
      UVALUE='Ok', $
      VALUE='Ok')

  WIDGET_CONTROL, GRIDAREA, /REALIZE

  LOADCT, 12    ;Load an appropriate colortable.
  black = 0*16 & white = 15*16  ;Match color indices to colors we want to use.
  red = 10*16 & lt_green=8*16
  lt_blue = 5*16 & dk_blue=6*16

  A = FINDGEN(16) * (!PI*2/16.)                         ; Make a vector of 16 points, A(i) = 2pi/16.
  USERSYM, COS(A)/2.8, SIN(A)/2.8, /FILL,COLOR=red      ; Define the symbol to be a unit circle with 16 points. Set the filled flag.

  PLOT,quake_x,$
       quake_y, $
       /YNOZERO,$
       BACKGROUND=lt_blue,$
       COLOR=black,$
       XCHARSIZE=1,$
       YCHARSIZE=1,$
       PSYM=8,$
       LINESTYLE=0,$
       XTITLE='Easting',$
       YTITLE='Northing',$
       /DEVICE,POS=plotpos,$
       XRANGE=[limits(0),limits(1)],$
       YRANGE=[limits(2),limits(3)],$
       XSTYLE=1,$
       YSTYLE=1

IF NOT KEYWORD_SET(POINT_MODE) THEN BEGIN

   IF KEYWORD_SET(AREA) THEN BEGIN
      PLOTS,area(0),area(2),COLOR=black, /CONTINUE
      PLOTS,area(1),area(2),COLOR=black, /CONTINUE
      PLOTS,area(1),area(3),COLOR=black, /CONTINUE
      PLOTS,area(0),area(3),COLOR=black, /CONTINUE
      PLOTS,area(0),area(2),COLOR=black, /CONTINUE
      PLOTS,area(0),area(2),COLOR=black
   ENDIF

   IF KEYWORD_SET(GRID_X) THEN BEGIN

      IF NOT KEYWORD_SET(SUPRESS_GRID) THEN BEGIN

         tmp1=FLTARR(N_ELEMENTS(grid_x)*N_ELEMENTS(grid_y))
         tmp2=tmp1
         for i=0,N_ELEMENTS(grid_x)-1 do begin
             for j=0,N_ELEMENTS(grid_y)-1 do begin
                 tmp1(i*N_ELEMENTS(grid_y)+j)=grid_x(i)
             endfor
             tmp2(i*N_ELEMENTS(grid_y):(i+1)*N_ELEMENTS(grid_y)-1)=grid_y
         endfor

         OPLOT,PSYM=1,tmp1,tmp2,COLOR=dk_blue
      ENDIF
   ENDIF

ENDIF

IF KEYWORD_SET(MAINSHOCKS) THEN BEGIN
   X = [-1, -2, 0, 2, 1,2,0,-2]
   Y = [0, -2, -1, -2, 0,2,1,2]
   USERSYM,X,Y,COLOR=dk_blue,/FILL
   OPLOT,xmain,ymain,PSYM=8
ENDIF

Ev=''

WHILE (Ev NE 'Cancel') AND (Ev NE 'Ok') DO BEGIN

  Event=WIDGET_EVENT(GRIDAREA)

  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  CASE Ev OF

  'Select Area': BEGIN
     IF KEYWORD_SET(POINT_MODE) THEN BEGIN
        CURSOR,x1,y1,1,/DEVICE
        x2=x1
        y2=y1
     ENDIF ELSE BEGIN
        BOXAREA,x1,y2,x2,y1,FLAG=flag
        ENDELSE
     END
  'PICK_MAINSHOCK': BEGIN
     PK, xmain, ymain, iselect
     SHOCKINFO,info_shock,$
               iselect,$
               FOCUS=focus,$
               SUPRESS_SWITCH=SUPRESS_SWITCH,$
               SHOCKINFOTEXT=shockinfotext
     selected_shock =  iselect
     END
  'Cancel': BEGIN
      END
  'Ok': BEGIN
      END
  ENDCASE

ENDWHILE

WIDGET_CONTROL,GRIDAREA,/DEST
IF Ev EQ 'Ok' THEN BEGIN
   IF KEYWORD_SET(SUPRESS_SELECT) THEN RETURN,-1 ELSE IF DEFINED(x1)THEN RETURN,IPOLBOX(plotpos,limits,[x1,x2,y1,y2]) ELSE RETURN,-1
ENDIF ELSE RETURN,-1

END




