;-------------------------------------------------------------  
;+  
; NAME:  
;       MAP_QUAKE  
; PURPOSE:  
;       Sets up a mercator map and overplots it with earthquake symbols  
; MAIN PROGRAM:
;       SLAP - program for testing SEISMOLAP 
;       SEISMOLAP was developed by Jochen Zschau,
;       Geoforschungszentrum Potsdam, Germany
; CALLING SEQUENCE:  
;       MAP_QUAKE,qu_lon,qu_lat[,LIMITS=limits][,GRID=GRID][,LABEL=LABEL][,HIRES=HIRES]  
; INPUTS:  
;       QU_LON = Vektor of longitudes of earthquakes. Value given in degree.degree  
;       QU_LAT = Vektor of latitudes of earthquakes. Value given in degree.degree  
; KEYWORD PARAMETERS:  
;       LIMITS = Vewktor of user-defined limits=[latmin,lonmin,latmax,lonmax]  
;                If LIMITS=limits is set, but limits is undefined, the borders of  
;                the earthquakes are returned  
; MODIFICATION HISTORY:  
;       Matthias Liebetanz, TH-Darmstadt, Germany, 27 April, 1998.  
;-  
;-------------------------------------------------------------  
PRO MAP_QUAKE,qu_lat,qu_lon,LIMITS=limits,GRID=GRID,LABEL=LABEL,HIRES=HIRES  
  
;  WIDGET_CONTROL,/HOUR  
  LOADCT, 12    ;Load an appropriate colortable.  
  black = 0*16 & white = 15*16  ;Match color indices to colors we want to use.  
  red = 10*16 & lt_green=3*16  
  lt_blue = 5*16 & dk_blue=6*16  
  ERASE, lt_blue        ;Erase the background to light blue.  
  
  IF NOT KEYWORD_SET(LIMITS) THEN BEGIN  
     limits=DBLARR(4)  
     limits(0)=MIN(qu_lon,MAX=tmp) & limits(1)=tmp  
     limits(2)=MIN(qu_lat,MAX=tmp) & limits(3)=tmp  
  ENDIF  
  latsize=limits(3)-limits(2) & lonsize=limits(1)-limits(0)  
  IF lonsize/2 GT latsize THEN BEGIN  
     limits(0)=limits(0)-(lonsize/2-latsize)/2  
     limits(2)=limits(2)+(lonsize/2-latsize)/2  
  ENDIF ELSE BEGIN  
     limits(1)=limits(1)-(latsize-lonsize/2)/2  
     limits(3)=limits(3)+(latsize-lonsize/2)/2  
  ENDELSE  
  IF lonsize/2 GT latsize THEN BEGIN  
     limits(2)=limits(2)-(lonsize/2-latsize)/2  
     limits(3)=limits(3)+(lonsize/2-latsize)/2  
  ENDIF ELSE BEGIN  
     limits(0)=limits(0)-(latsize-lonsize/2)/2  
     limits(1)=limits(1)+(latsize-lonsize/2)/2  
  ENDELSE  
  
  IF KEYWORD_SET(HIRES) THEN BEGIN 
     MAP_SET, $  
         GRID=GRID,$  
         COLOR=black,$  
         /MERC,$  
         LIMIT=[limits(2),limits(0),limits(3),limits(1)],$  
         /NOERASE,$  
         LABEL=LABEL,$  
         XMARGIN=[0.0,0.0],YMARGIN=[0.0,0.0], $  
         HIRES=HIRES  
 
 
     ;Fill the continent boundaries with solid white.  
     MAP_CONTINENTS, /FILL_CONTINENTS, COLOR=white, HIRES=HIRES  
     ;Overplot coastline data.  
     MAP_CONTINENTS, /COASTS, COLOR=black, MLINETHICK=0.3, HIRES=HIRES  
     ;Add rivers, in dark blue.  
     MAP_CONTINENTS, /RIVERS, COLOR=dk_blue, MLINETHICK=0.4, HIRES=HIRES  
     ;Show national borders.  
     MAP_CONTINENTS, /COUNTRIES, COLOR=red, MLINETHICK=1.5, HIRES=HIRES  
   
  ENDIF ELSE BEGIN 
     MAP_SET, $  
         GRID=GRID,$  
         /MERC,$  
         LIMIT=[limits(2),limits(0),limits(3),limits(1)],$  
         /NOERASE,$  
         LABEL=LABEL,$  
         XMARGIN=[0.0,0.0],YMARGIN=[0.0,0.0], $  
   
     ;Fill the continent boundaries with solid white.  
     MAP_CONTINENTS, COLOR=white 
     ;Overplot coastline data.  
     MAP_CONTINENTS, COLOR=black, MLINETHICK=0.3 
     ;Add rivers, in dark blue.  
     MAP_CONTINENTS, COLOR=dk_blue, MLINETHICK=0.4 
     ;Show national borders.  
     MAP_CONTINENTS, COLOR=red, MLINETHICK=1.5 
  ENDELSE 
  
  A = FINDGEN(16) * (!PI*2/32.)                      ; Make a vector of 16 points, A(i) = 2pi/16.  
  USERSYM, COS(A)/2., SIN(A)/2., /FILL,COLOR=red           ; Define the symbol to be a unit circle with 16 points. Set the filled flag.  
  !P.CHARSIZE=3                                         ; Scaling factor is 2  
  !X.TICKS=2  &  !Y.TICKS=2                             ; Suppress tick marks  
  
  OPLOT, qu_lon,qu_lat,PSYM=8  
END  
  
