;------------------------------------------------------------- 
;+ 
; NAME: 
;       REWRITE_STRUCT 
; PURPOSE: 
;       Redefines type and size of a selected tag 
; CATEGORY: 
;       Structures 
; CALLING SEQUENCE: 
;       result=REWRITE_STRUCT(struct,TAG_NAME=TAG_NAME,NEW_VALUE=NEW_VALUE) 
; INPUTS: 
;       struct = The structure to be rewritten 
;       TAG_NAME = The selected tag 
;       NEW_VALUE = The new value of TAG_NAME. It can be of other type and size. The old information is erased. 
; KEYWORD PARAMETERS: 
; OUTPUTS: 
;       Returnes new structure 
; COMMON BLOCKS: 
; NOTES: 
; MODIFICATION HISTORY: 
;       Matthias Liebetanz, TH-Darmstadt, Germany, 10 Dec, 1997. 
; 
;- 
;------------------------------------------------------------- 
FUNCTION REWRITE_STRUCT,struct,TAG_NAME=tag_name,NEW_VALUE=new_value 
 
 
IF DEFINED(struct,TAG=tag_name,/STRUCT) THEN BEGIN ; Check if struct.tag is defined 
   tag_name=UPCASE(tag_name)                       ; Make sure there are all letters of upper case 
   tagnames=TAG_NAMES(struct) 
   index=WHERE(tagnames NE tag_name,count)         ; All tags will be kept with exception of selected tag 
   IF count NE 0 THEN BEGIN 
      name=tagnames(index(0)) 
      value=struct.(index(0)) 
      new_struct=CREATE_STRUCT(name,value)         ; Initialize new_struct 
      FOR i=1,N_ELEMENTS(index)-1 DO BEGIN         ; Copy the old tags in new structure 
          name=tagnames(index(i)) 
          value=struct.(index(i)) 
          new_struct=CREATE_STRUCT(new_struct,name,value) 
      ENDFOR 
      new_struct=CREATE_STRUCT(new_struct,tag_name,new_value)    ; At least, copy the new value 
   ENDIF ELSE RETURN, -1 
 
ENDIF ELSE RETURN, -1 
RETURN,TEMPORARY(new_struct) 

END 
