;-------------------------------------------------------------
;+
; NAME:
;       SREPLACE
; PURPOSE:
;       Replace substrings in a string-array.
; CATEGORY:
;       String processing
; CALLING SEQUENCE:
;       sreplace, dest, old, new, count, i[,UPCASE=UPCASE][,FIRSTPOS=firstpos][,LASTPOS=lastpos]
; INPUTS:
;       DEST=String-Array to be examined
;       OLD=Search-string
;       NEW=replaces OLD
; KEYWORD PARAMETERS:
;       Keywords:
;       COUNT=counts the number of occurences of OLD in DEST
;       UPCASE=The search-string is also forced to uppercase
;       I=An array with the indexes of the occurences
; OUTPUTS:
;       COUNT=counts the number of occurences of OLD in DEST
;       I=An Array with the indexes of the occurences
; COMMON BLOCKS:
; NOTES:
;       USES DEFINED.PRO
; MODIFICATION HISTORY:
;       Matthias Liebetanz, 31 Mrz, 1997
;-
;-------------------------------------------------------------
PRO SREPLACE,dest,old,new,FIRSTPOS=firstpos,LASTPOS=lastpos,UPCASE=UPCASE,count,index

IF N_ELEMENTS(firstpos) NE 0 THEN BEGIN
   IF firstpos EQ 0 THEN BEGIN
      PRINT, 'Value 0 for FIRSTPOS not allowed.'
      GOTO,_END
   ENDIF
ENDIF

IF N_ELEMENTS(lastpos) NE 0 THEN BEGIN
   IF lastpos EQ 0 THEN BEGIN
      PRINT, 'Value 0 for LASTPOS not allowed.'
      GOTO,_END
   ENDIF
ENDIF

IF N_ELEMENTS(new) NE  N_ELEMENTS(old) THEN BEGIN
   PRINT, 'NEW-String must be of same type and dimension as OLD-String'
   GOTO, _END
ENDIF

IF KEYWORD_SET(UPCASE) THEN $
   BEGIN
   tmp_old=[STRLOWCASE(old),STRUPCASE(old)]
   tmp_new=[STRLOWCASE(new),STRUPCASE(new)]
ENDIF ELSE BEGIN
   tmp_old=old
   tmp_new=new
ENDELSE

IF KEYWORD_SET(FIRSTPOS) THEN BEGIN
   firstlength=firstpos
   IF KEYWORD_SET(LASTPOS) THEN BEGIN
      secondlength=lastpos+1-firstpos
      firstsubstr=STRMID(dest,0,firstlength)
      secondsubstr=STRMID(dest,firstpos,secondlength)
      thirdsubstr=STRMID(TEMPORARY(dest),firstpos+secondlength,10000)
      dest=TEMPORARY(secondsubstr)
   ENDIF ELSE BEGIN
      firstsubstr=STRMID(dest,0,firstlength)
      dest=STRMID(TEMPORARY(dest),firstpos,10000)
   ENDELSE
   ENDIF ELSE BEGIN
   IF KEYWORD_SET(LASTPOS) THEN BEGIN
      firstlength=lastpos+1
      firstsubstr=STRMID(dest,0,firstlength)
      secondsubstr=STRMID(TEMPORARY(dest),lastpos+1,10000)
      dest=TEMPORARY(firstsubstr)
   ENDIF
ENDELSE

count=0
n_old=N_ELEMENTS(tmp_old)
iter=N_ELEMENTS(dest)                                   ; number of elements in 'DEST'
FOR i=0,iter-1 DO BEGIN                                 ; FOR :Search and replace occurences of '_old' with '_new'
    FOR j=0,n_old-1 DO BEGIN                              ; Search for each element of 'OLD'
        _old=tmp_old(j)
        _new=tmp_new(j)
        len_old=STRLEN(_old)                                  ; Length of the '_old'-string
        frst=STRPOS(dest(i),_old)                             ; Look for first occurence of '_old' in 'DEST'
        IF frst NE -1 THEN BEGIN                              ; IF an occurence is found
           lst=RSTRPOS(dest(i),_old)                            ; the last position of '_old' in 'DEST'
           _dest=STR_SEP(dest(i),_old)                          ; Divide DEST into pieces as designed by the '_old'-string
           n_occur=N_ELEMENTS(_dest)-1                          ; count the number of occurences in the row
           count=count+n_occur                                  ; increment the number of occurences in _old
           IF DEFINED(index) THEN index=[index,i] ELSE index=i  ; add a _new row to INDEX
           len_dest=STRLEN(dest(i))                             ; Length of the row
           lst_pssble=len_dest-len_old-1                        ; the last possible position of '_old' in 'DEST
           tmp=''                                                  ; intitialize tmp
           IF frst NE 0 THEN BEGIN                              ; If '_old' occures not at the beginning of 'DEST'
              FOR k=0,n_occur-1 DO tmp=tmp+_dest(k)+_new
              IF lst_pssble NE lst THEN tmp=tmp+_dest(k)
           ENDIF ELSE BEGIN                                    ; If '_old' occures at the beginning of 'DEST'
              FOR k=0,n_occur-1 DO tmp=tmp+_new+_dest(k+1)
              IF lst_pssble EQ lst THEN tmp=tmp+_new
           ENDELSE
           dest(i)=tmp
        ENDIF
    ENDFOR
 ENDFOR

IF KEYWORD_SET(FIRSTPOS) THEN BEGIN
   IF KEYWORD_SET(LASTPOS) THEN $
      dest=TEMPORARY(firstsubstr)+TEMPORARY(dest)+TEMPORARY(thirdsubstr) $
   ELSE $
      dest=TEMPORARY(firstsubstr)+TEMPORARY(dest)
ENDIF ELSE BEGIN
   IF KEYWORD_SET(LASTPOS) THEN $
      dest=TEMPORARY(dest)+TEMPORARY(secondsubstr)
ENDELSE

_END:
END

