;-------------------------------------------------------------
;+
; NAME:
;       RENAMER
; PURPOSE:
;       Search and replaces strings in selected files.
; CALLING SEQUENCE:
;       Renamer
; MODIFICATION HISTORY:
;       Matthias Liebetanz, TH-Darmstadt, Germany, 04 Apr, 1997
;-
;-------------------------------------------------------------

PRO PDMENU22_Event, Event


  CASE Event.Value OF


  'File.Exit': BEGIN
    PRINT, 'Event for File.Exit'
       WIDGET_CONTROL, Event.Top,  /DESTROY                       ; Destroy application
       XMANAGER
   END
  'Renaming.Search and replace in files': BEGIN
    PRINT, 'Event for Renaming.Search and replace in files'

    state_holder = WIDGET_INFO( event.handler, /CHILD )
    WIDGET_CONTROL, state_holder, GET_UVALUE=statev                  ; Get the state
    IF DEFINED(state,/STRUCTURE) THEN $
      path=state.path $
    ELSE $
      state=CREATE_STRUCT('path','')

     Filename=' '
     WHILE FileName NE '' DO BEGIN
      FileName = PICKFILE(/READ, GROUP=event.top,             $
                           TITLE='Choose one file of directory to be examined', $
                           FILTER='*.pro', $
                           PATH=state.path,  $
                           GET_PATH=path)
      IF FileName EQ '' THEN GOTO, _End_Search
      state.path=path
      WIDGET_CONTROL,/HOURGLASS

      searchedfiles = FINDFILE(path+'*.pro',COUNT=n_searchedfiles)

      IF n_searchedfiles EQ 0 THEN GOTO,_End_Search            ; Cancel 'File.Search..'

      XREPLACE,ALTERED=altered, $
              SEARCH_STRING = search, $
              REPLACE_STRING = replace

     IF NOT altered THEN GOTO, _End_Search


      text=[['This may change files on your hard disk!'],['Do you want to continue?']]
      yn=XDECISION(text)

      IF yn EQ 'N' THEN GOTO, _End_Search $                         ; Terminated by user

       ELSE BEGIN

       SREPLACE,/UPCASE,search,'.pro',''                             ; Delete extensions from search-strings
       SREPLACE,/UPCASE,replace,'.pro',''

       tmp=''

       n_equal=N_ELEMENTS(search)

       progress=CW_PROGRESS(event.handler,TITLE='Search in files')       ; Calling CW_PROGRESS for Information about progress of searching

       tot_repl=0
       FOR i=0,n_searchedfiles-1 DO BEGIN
        current_file=searchedfiles(i)
        OPENR,UNIT,current_file,/GET_LUN,ERROR=io_err
        IF io_err lt 0 THEN $                                            ; OK?
         mssg = WIDGET_MESSAGE( STRING(!err_string) + $                  ; NO
                ' Cannot open ' + current_file, /ERROR) $
         ELSE BEGIN
         file=''
         WHILE NOT EOF(UNIT)  DO BEGIN                                  ; Read the file in the variable 'file', its lines in 'temp'
          READF,UNIT, tmp, $
                      FORMAT='(A)'
          file=[file,tmp]
          ENDWHILE
          original=file

          SREPLACE,file,search,replace,/UPCASE,count,index

          IF count NE 0 THEN BEGIN                                      ; Search-String was found in file
           tot_repl=tot_repl+1                                          ; Count total number of replaced files
           FREE_LUN,UNIT
           old_file=current_file                                        ; initialize
           SREPLACE,old_file,'.pro','.old'                        ; Rename original file in *.old
           OPENW,UNIT,old_file,GET_LUN,ERROR=io_err
           IF io_err lt 0 THEN $                                        ; I/O OK?
            mssg = WIDGET_MESSAGE( STRING(!err_string) + $              ; NO
                   ' Cannot open ' + current_file, /ERROR) $
            ELSE PRINTF,UNIT,original(1:*), FORMAT='(A)'                ; Save original file as *.old
            FREE_LUN,UNIT
            OPENW,UNIT,current_file,GET_LUN,ERROR=io_err
            IF io_err lt 0 THEN $                                       ; I/O OK?
             mssg = WIDGET_MESSAGE( STRING(!err_string) + $             ; NO
                   ' Cannot open ' + current_file, /ERROR) $
            ELSE BEGIN
             PRINTF,UNIT,file(1:*), FORMAT='(A)'                        ; Save new file as *.pro
             ENDELSE
            FREE_LUN,UNIT
           ENDIF
          FREE_LUN,UNIT
         ENDELSE


         valid=WIDGET_INFO(progress,/VALID_ID)                           ; Look, if the CANCEL-button was pressed

         IF valid NE 0 THEN $
          WIDGET_CONTROL,progress, $                                     ; Update the information about progress
                         SET_VALUE=STRTRIM(STRING(i+1),1)+' of '+STRTRIM(STRING(n_searchedfiles),1)+' Files examined - '+STRTRIM(STRING(tot_repl),1)+' replaced.', $
                         /NO_COPY $

          ELSE GOTO, _End_Search                                        ; CW_PROGRESS is already destroyed

         event=WIDGET_EVENT(/nowait)                                    ; Look for arrived events and process them (f.e.CANCEL-Button)

        ENDFOR

      ENDELSE

      valid=WID_ALIVE(progress,/DESTROY)
      ENDWHILE


     _End_Search:

    valid=WID_ALIVE(progress,/DESTROY)

    END
  'Renaming.View results': BEGIN
    PRINT, 'Event for Renaming.View results'

      state_holder = WIDGET_INFO( event.handler, /CHILD )
      WIDGET_CONTROL, state_holder, GET_UVALUE=state,/NO_COPY             ; Get the state

      def_state=DEFINED(state, /STRUCTURE)
      def_state_newnames=DEFINED(state, TAG='NEWNAMES', /STRUCTURE)

      IF def_state EQ 0 THEN BEGIN                                          ; If it is not a structure
       mssg=WIDGET_MESSAGE('No file open. Open File first.',/INFORMATION,TITLE='Renamer')
       GOTO, _End_Show_Results                                   ; CANCEL 'File.Save File as..'
      ENDIF ELSE $
       IF def_state_newnames EQ 0 THEN BEGIN                     ; If it is a structure without tag: 'newnames'
        mssg=WIDGET_MESSAGE('Please truncate Filenames first.',/INFORMATION,TITLE='Renamer')
        GOTO, _End_Show_Results                                  ; CANCEL 'File.Save File as..'
        ENDIF

      Text = ['examined:'+STRING(state.n_oldnames),             $
              'renamed:'+STRING(state.n_renamed),         $
              'equal'+STRING(state.n_equal)]
      IF state.n_equal NE 0 THEN $                               ; If there are equal filenames
      Text = [Text,                                       $
              '',                                         $
              'Result equal:',                            $
              STRING(state.oldnames(state.equal_index) )+'  -->  '+ $
              STRING(state.newnames(state.equal_index) )]

      Text = [Text,                                       $
              '',                                         $
              'Result renamed:',                          $
              STRING(state.oldnames(state.new_index))+'  -->  '+ $
              STRING(state.newnames(state.new_index))]

      CW_ShowText,event.top,Text,TITLE='Results'


    _End_Show_Results:
    WIDGET_CONTROL, state_holder, SET_UVALUE=state,/NO_COPY             ; store the state

    END

  'Help.About Renamer for Windows..': BEGIN
    PRINT, 'Event for Help.About Renamer for Windows..'
    mssg=WIDGET_MESSAGE(['Author: Matthias Liebetanz, TH-Darmstadt, Germany.',$
                         'The Author makes no warranty that this program is free of error.'],$
                         /INFORMATION,TITLE='About Renamer for Windows, Vers 1.0')

    END
  ENDCASE
END


; DO NOT REMOVE THIS COMMENT: END PDMENU22
; CODE MODIFICATIONS MADE BELOW THIS COMMENT WILL BE LOST.








; CODE MODIFICATIONS MADE ABOVE THIS COMMENT WILL BE LOST.
; DO NOT REMOVE THIS COMMENT: BEGIN MAIN13

PRO MAIN13_Event, Event


  WIDGET_CONTROL,Event.Id,GET_UVALUE=Ev

  CASE Ev OF

  ; Event for PDMENU22
  'PDMENU22': PDMENU22_Event, Event
  ENDCASE
END


; DO NOT REMOVE THIS COMMENT: END MAIN13
; CODE MODIFICATIONS MADE BELOW THIS COMMENT WILL BE LOST.



PRO RENAMER, GROUP=Group


  IF N_ELEMENTS(Group) EQ 0 THEN GROUP=0

  junk   = { CW_PDMENU_S, flags:0, name:'' }


  MAIN13 = WIDGET_BASE(GROUP_LEADER=Group, $
      ROW=1, $
      MAP=1, $
      TITLE='Renamer', $
      UVALUE='MAIN13')


  BASE7 = WIDGET_BASE(MAIN13, $
      ROW=1, $
      MAP=1, $
      UVALUE='StateHolder')

  MenuDesc159 = [ $
      { CW_PDMENU_S,       1, 'File' }, $ ;        0
        { CW_PDMENU_S,       2, 'Exit' }, $ ;        5
      { CW_PDMENU_S,       1, 'Renaming' }, $ ;        6
        { CW_PDMENU_S,       0, 'Search and replace in files' }, $ ;        9
        { CW_PDMENU_S,       2, 'View results' }, $ ;        10
      { CW_PDMENU_S,       3, 'Help' }, $ ;        11
        { CW_PDMENU_S,       2, 'About Renamer for Windows..' } $  ;      12

  ]


  PDMENU22 = CW_PDMENU( MAIN13, MenuDesc159, /RETURN_FULL_NAME, $
      UVALUE='PDMENU22')

  WIDGET_CONTROL, MAIN13, /REALIZE

  XMANAGER, 'MAIN13', MAIN13
END



